__title__ = "eciespy"
__version__ = "0.3.4"
__description__ = "Elliptic Curve Integrated Encryption Scheme for secp256k1 in Python"
__url__ = "https://github.com/ecies/py"
__author__ = "Weiliang Li"
__author_email__ = "to.be.impressive@gmail.com"
__license__ = "MIT"
